//-----------------------------------------------------------------------------
/// Modulename : BinarizeView.cpp @file
///
/// Interface  : BinarizeView.h
///
///              Implementation of class CBinarizeView.
///
///  2003 Sirona Dental Systems GmbH - All Rights Reserved
///
/// Status     : Draft
///
/// Security   : -
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Standard includes
//-----------------------------------------------------------------------------
#include "stdafx.h"

//-----------------------------------------------------------------------------
// Own includes
//-----------------------------------------------------------------------------
#include "resource.h"
#include "Binarize.h"
#include "BinarizeDoc.h"
#include "BinarizeView.h"

//-----------------------------------------------------------------------------
// Private definitions, -macros and -constants
//-----------------------------------------------------------------------------
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//-----------------------------------------------------------------------------
// Private data types
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// External references
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Module global variables
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Prototype of the private functions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Implementation
//-----------------------------------------------------------------------------
IMPLEMENT_DYNCREATE(CBinarizeView, CView)

BEGIN_MESSAGE_MAP(CBinarizeView, CView)
   //{{AFX_MSG_MAP(CBinarizeView)
   ON_COMMAND(IDM_GETACTIVEIMAGE, OnGetactiveimage)
   ON_COMMAND(IDM_FILTER, OnFilter)
   ON_COMMAND(IDM_SETACTIVEIMAGE, OnSetactiveimage)
   //}}AFX_MSG_MAP
END_MESSAGE_MAP()

//-----------------------------------------------------------------------------
/// Constructor
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
CBinarizeView::CBinarizeView()
{
}

//-----------------------------------------------------------------------------
/// Destructor
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
CBinarizeView::~CBinarizeView()
{
   if (m_pdispApp)
   {
      (void) m_pdispApp->Release();
      m_pdispApp = NULL;
   }
}

//-----------------------------------------------------------------------------
/// Retrieves the running SIDEXIS XG instance
///
/// @return    Dispatch of running SIDEXIS XG instance
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
LPDISPATCH CBinarizeView::GetApplication()
{
   SCODE        sc = S_OK;
   CLSID        clsid;
   IApplication ISidexis;
   LPUNKNOWN    pUnkSrv  = NULL;
   LPDISPATCH   pdispApp = NULL;

   // Retrieve ClassID
   sc = CLSIDFromProgID( L"SIDEXISNG.Application", &clsid );
   
   if (FAILED( sc ))
      return NULL;

   // Find the active object
   sc = GetActiveObject( clsid, NULL, &pUnkSrv );  // returns Unknown pointer

   if (FAILED( sc ))
      return NULL;

   // Get the Dispatch interface
   sc = pUnkSrv->QueryInterface( IID_IDispatch, (void**) &pdispApp );   // returns the dispatch Interface

   if (FAILED( sc ))
   {
      (void) pUnkSrv->Release();
      return NULL;
   }

   // We don't need this anymore
   (void) pUnkSrv->Release();

   return pdispApp;
}

//-----------------------------------------------------------------------------
/// This is called the first time this View is updated
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
void CBinarizeView::OnInitialUpdate() 
{
   CView::OnInitialUpdate();
   
   m_pdispApp = GetApplication();
   if (!m_pdispApp)
   {
      AfxMessageBox( IDS_E_SIDEXISNOTRUNNING, MB_ICONSTOP | MB_OK );
      return;
   }
   m_SidexisXG.AttachDispatch( m_pdispApp, FALSE );
}

//-----------------------------------------------------------------------------
/// Draw method
///
/// @param     pDC [IN] Device context
///
/// Remarks  : -
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
void CBinarizeView::OnDraw(CDC* pDC) 
{
   HBITMAP hbmp = m_Image.GetBitmap();
   if (!hbmp)
      return;

   CDC dcMemory;
   dcMemory.CreateCompatibleDC( pDC );

   CBitmap* pBitmap = CBitmap::FromHandle( hbmp );
   CBitmap* pOld = dcMemory.SelectObject( pBitmap );

   // Copy the bits from the in-memory DC into the on-screen DC to actually do the painting
   pDC->BitBlt( 0, 0, m_Image.GetWidth(), m_Image.GetHeight(), &dcMemory, 0, 0, SRCCOPY);

   // Display the calculated ROI
   CBrush br;
   br.CreateSolidBrush( RGB( 255, 0, 0 ) );
   pDC->FrameRect( m_Image.GetROI(), &br );

   dcMemory.SelectObject( pOld );
}

//-----------------------------------------------------------------------------
/// Gets the active image data from SIDEXIS XG
///
/// Remarks  : Please note: This method is only intended to be an example.
///            You can't generate new views of images by changing the image
///            pixel map directly without being run as plugin.
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
void CBinarizeView::OnGetactiveimage() 
{
   if (!m_pdispApp)
      return;

   LPDISPATCH pdispExam = m_SidexisXG.GetActiveExam();
   if (!pdispExam)
   {
      AfxMessageBox( IDS_E_NOACTIVEEXAM, MB_ICONSTOP | MB_OK );
      return;
   }

   IExam Exam;
   Exam.AttachDispatch( pdispExam );

   LPDISPATCH pdispTakeFrame = Exam.GetActiveTakeFrame();

   if (!pdispTakeFrame)
   {
      AfxMessageBox( IDS_E_NOACTIVETAKEFRAME, MB_ICONSTOP | MB_OK );
      return;
   }

   ITakeFrame TakeFrame;
   TakeFrame.AttachDispatch( pdispTakeFrame );

   LPDISPATCH pdispImage = TakeFrame.GetImage();

   if (!pdispImage)
      return;

   if (!m_Image.FromSidexis( pdispImage ))
   {
      AfxMessageBox( IDS_E_GETACTIVEIMAGE, MB_ICONSTOP | MB_OK );
      return;
   }

   (void) pdispImage->Release();

   Invalidate();
}

//-----------------------------------------------------------------------------
/// Filters the image data
///
/// Remarks  : Please note: This method is only intended to be an example.
///            You can't generate new views of images by changing the image
///            pixel map directly without being run as plugin.
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
void CBinarizeView::OnFilter() 
{
   if (!m_Image.GetBinarizeParameters())
      return;

   CWaitCursor wait;
   if (!m_Image.Binarize())
      return;

   Invalidate();
}

//-----------------------------------------------------------------------------
/// Sets the pixel data back to SIDEXIS XG
///
/// Remarks  : Please note: This method is only intended to be an example.
///            You can't generate new views of images by changing the image
///            pixel map directly without being run as plugin.
///
/// Security : -
///
/// Status   : Draft
///
/// Revision history - Version 1.0
///
/// 15.09.03 - Draft => V.1.0 (MCI, Fa. MGW)
//-----------------------------------------------------------------------------
void CBinarizeView::OnSetactiveimage() 
{
   if (!m_pdispApp)
      return;

   LPDISPATCH pdispExam = m_SidexisXG.GetActiveExam();
   if (!pdispExam)
   {
      AfxMessageBox( IDS_E_NOACTIVEEXAM, MB_ICONSTOP | MB_OK );
      return;
   }

   IExam Exam;
   Exam.AttachDispatch( pdispExam );

   LPDISPATCH pdispTakeFrame = Exam.GetActiveTakeFrame();

   if (!pdispTakeFrame)
   {
      AfxMessageBox( IDS_E_NOACTIVETAKEFRAME, MB_ICONSTOP | MB_OK );
      return;
   }

   ITakeFrame TakeFrame;
   TakeFrame.AttachDispatch( pdispTakeFrame );

   LPDISPATCH pdispImage = TakeFrame.GetImage();

   if (!pdispImage)
      return;

   if (!m_Image.ToSidexis( pdispImage ))
   {
      AfxMessageBox( IDS_E_SETACTIVEIMAGE, MB_ICONSTOP | MB_OK );
      return;
   }

   (void) pdispImage->Release();

   Exam.Invalidate();
}
